<?php
/**
 * Search & Filter Pro Visual Composer
 *
 * @package   Search_Filter
 * @author    Ross Morsali
 * @link      http://www.designsandcode.com/
 * @copyright 2015 Designs & Code
 *
 * @wordpress-plugin
 * Plugin Name:       Search & Filter Pro VC
 * Plugin URI:        http://www.designsandcode.com/wordpress-plugins/search-filter-pro/
 * Description:       This is a bridge plugin that allows easy usage of the Visual Composer Post Grid, and Masonry Post Grid widgets with Search & Filter
 * Version:           1.0.0
 * Author:            Designs & Code
 * Author URI:        http://www.designsandcode.com/
 * Text Domain:       search-filter
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.html
 * Domain Path:       /languages
 */

/*----------------------------------------------------------------------------*
 * Public-Facing Functionality
 *----------------------------------------------------------------------------*/


/*
 * Register hooks that are fired when the plugin is activated or deactivated.
 * When the plugin is deleted, the uninstall.php file is loaded.
 */
/*register_activation_hook( __FILE__, array( 'Search_Filter', 'activate' ) );
register_deactivation_hook( __FILE__, array( 'Search_Filter', 'deactivate' ) ); */

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-pdf-creator-activator.php
 */

add_action( 'plugins_loaded', array( 'Search_Filter_VC', 'get_instance' ) );

global $search_filter_vc_id;
$search_filter_vc_id = 0;

Class Search_Filter_VC
{
	protected static $instance = null;
	
	private $search_form_options = array();
	private $vc_shortcodes = array();
	
	public static function get_instance() {

		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self;
		}

		return self::$instance;
	}
	
	
	private function __construct()
	{
		// Check if Visual Composer is installed
        if ( ( ! defined( 'WPB_VC_VERSION' ) ) || ( ! defined( 'SEARCH_FILTER_VERSION' ) ) ) {
            // Display notice that Visual Compser is required
            //add_action('admin_notices', array( $this, 'showVcVersionNotice' ));
            return;
        }
		else
		{
			add_action( 'vc_before_init', array($this, 'init_vc_mappings') );
			add_action( 'init', array($this, 'vc_add_param_to_grid_settings'), 2000 );
			add_action( 'vc_after_init', array($this, 'vc_edit_mappings'), 2000 );
			//add_filter( 'vc_mapper_attribute_loop', array($this, 'filter_vc_post_grid'), 10, 2 );
			
			//add_filter( 'vc_map_get_param_defaults', array($this, 'vc_map_get_param_defaults'), 10, 2 );
			add_filter( 'shortcode_atts_vc_basic_grid', array($this, 'vc_filter_frontend_shortcode'), 10 );
			add_filter( 'shortcode_atts_vc_masonry_grid', array($this, 'vc_filter_frontend_shortcode'), 10 );
			add_filter( 'sf_ajax_data_fields', array($this, 'sf_ajax_data_fields'), 10, 2 );
			//add_filter( 'search_filter_ajax_results', array($this, 'search_filter_ajax_results'), 10 );
			add_filter("search_filter_ajax_object_vc_results", array($this, 'search_filter_ajax_object_vc_results'), 10, 1);
			add_filter( 'sf_ajax_results_url', array($this, 'sf_ajax_results_url'), 10, 2 );
			add_filter( 'search_filter_admin_option_display_results', array($this, 'search_filter_admin_option_display_results'), 10, 2 );
			add_filter( 'search_filter_form_attributes', array($this, 'search_filter_form_attributes'), 10, 2 );
			
			
			//add_filter( 'vc_shortcodes_css_class', array($this, 'change_element_class_name'), 1000 );
			
			//add_shortcode("sfvc", array($this, "sfvc"));
			//add_shortcode("sfvc_button", array($this, "sfvc_button"));
			
			add_action('wp_enqueue_scripts', array($this, "enqueue_scripts"), 10);
			
			//add_action( 'wp_ajax_search_filter_vc_get_results', array($this, 'get_results') );
			//add_action( 'wp_ajax_nopriv_search_filter_vc_get_results', array($this, 'get_results') );
			
		}
	}
		
	/*public function get_results()
	{
		$this->get_vc_results(array('shortcode_id' => '1477793943938-289cb011-1905-9'));
		exit;
	}*/
	
	public function enqueue_scripts()
	{
		
			wp_register_script( 'search-filter-pro-vc', plugins_url( 'js/vc-frontend-search-filter.js', __FILE__ ), array( 'jquery' ), "1.0.0" );
			wp_localize_script( 'search-filter-pro-vc', 'SFVC_DATA', array( 'ajax_url' => admin_url( 'admin-ajax.php' ), 'home_url' => (home_url('/')) ));
			wp_enqueue_script( 'search-filter-pro-vc');
			
	}
	public function search_filter_form_attributes($attributes, $sfid)
	{
		if(isset($attributes['data-display-result-method']))
		{
			if($attributes['data-display-result-method']=="vc_post_grid")
			{
				$attributes['data-ajax-target'] = '.search-filter-results-'.$sfid;
				$attributes['data-ajax-data-type'] = 'json';
			}
		}
		return $attributes;
	}
	public function search_filter_admin_option_display_results($display_results_options)
	{
		$display_results_options['vc_post_grid'] = array(
            'label'         => __('Visual Composer Post Grid'),
            'description'   => 
				'<p>'.__('Search Results will displayed using one of Visual Composers post grid elements.', '').'</p>'.
				'<p>'.__('Remember to set the <strong>Data Source</strong> in your post grid to use this Search Form.', '').'</p>',
            'base'          => 'shortcode'
        );
		
		return $display_results_options;
	}
	public function sf_ajax_data_fields($data_type, $sfid)
	{
		global $searchandfilter;
		$display_result_as = $searchandfilter->get($sfid)->settings("display_results_as");
		$enable_auto_count = $searchandfilter->get($sfid)->settings("enable_auto_count");
		$auto_submit = $searchandfilter->get($sfid)->settings("auto_submit");
		
		if($display_result_as=="vc_post_grid")
		{
			if(($enable_auto_count==1)&&($auto_submit==1))
			{//if auto count & auto submit are enabled, then we must request a copy of the search form too (with the updated count numbers & fields)
				$data_type = "form,vc_results";
			}
			else
			{
				$data_type = "vc_results";
			}		
		}
		
		return $data_type;
	}
	
	public function search_filter_ajax_object_vc_results($ajax_object)
	{
		$data = array();
		
		if((isset($_GET['sfvc_page_id']))&&(isset($_GET['sfvc_style']))&&(isset($_GET['sfvc_shortcode_id'])))
		{
			$data["page_id"] = (int)$_GET['sfvc_page_id'];
			$data["style"] = esc_attr($_GET['sfvc_style']);
			$data["shortcode_id"] = esc_attr($_GET['sfvc_shortcode_id']);
						
			$ajax_object['results'] = $this->get_vc_results($data);
		}
		return $ajax_object;
	}
		
	public function sf_ajax_results_url($url, $sfid)
	{
		global $searchandfilter;
		$display_result_as = $searchandfilter->get($sfid)->settings("display_results_as");
		
		if($display_result_as=="vc_post_grid")
		{
			$url = add_query_arg("sfid", $sfid, $url);//add_query_arg("sfid","", admin_url( 'admin-ajax.php' ));
			$url = add_query_arg("sf_action", "get_data", $url);//add_query_arg("sfid","", admin_url( 'admin-ajax.php' ));
		}
		
		return $url;		
	}
		
	public function change_element_class_name($class_string, $tag) {
		
		if(($tag=="vc_basic_grid")||($tag=="vc_masonry_grid"))
		{
			//if(strpos($class_string, "vc_grid-container")!==false)
			//{
				global $search_filter_vc_id;
				return $class_string." search-filter-results search-filter-results-$search_filter_vc_id";
				$search_filter_vc_id = 0;
				
				remove_filter( 'vc_shortcodes_css_class', array($this, 'change_element_class_name'), 1000, 2 );
			//}
		}
		
		return $class_string;
	}
	public function sfvc_button($atts)
	{
		return "<button id='sfvc-load-results'>Load New Results</button>";
	}
	public function get_vc_results($atts)
	{
		$allowed = true;
        $tag = 'vc_basic_grid';
        $tag = 'vc_masonry_grid';
       // echo "TAG: ".$tag;
        if ( $allowed ) {
            $shortcode_fishbone = visual_composer()->getShortCode( $tag );

            if ( is_object( $shortcode_fishbone ) ) {
                /** @var $vc_grid WPBakeryShortcode_Vc_Basic_Grid */
                $vc_grid = $shortcode_fishbone->shortcodeClass();
                if ( method_exists( $vc_grid, 'isObjectPageable' ) && $vc_grid->isObjectPageable() && method_exists( $vc_grid, 'renderAjax' ) ) {
					
                    $data = array();
                    $data["visible_pages"]= "5";
                    //$data["page_id"] = (int)$_GET['sfvc_page_id'];
                    //$data["style"]= "all";
                    $data["action"]= "vc_get_vc_grid_data";
                    //$data["shortcode_id"]= "1477785226098-7a81b3b1-c8cd-9";
                    //$data["shortcode_id"]= "1477788327735-2bfcb830-29cb-9";
					
					if(!isset($atts['shortcode_id']))
					{
						$atts['shortcode_id'] = "";
						$atts['page_id'] = "";
						$atts['sfvc_style'] = "";
					}
					
                    $data["shortcode_id"]= $atts['shortcode_id'];
                    $data["page_id"]= $atts['page_id'];
                    $data["style"]= $atts['sfvc_style'];
					
                    $data["tag"]= $tag;

                    //echo "<button id='sfvc-load-results'>Load New Results</button>";
					
					//var_dump($data);
                    return $vc_grid->renderAjax($data);
                    
                    //die();
                }
            }
        }
	}
	
	public function vc_filter_frontend_shortcode($atts)
	{
		$atts['el_class'] = "";
		if($atts['post_type']=='search_filter')
		{//then the user selected S&F as the input source
			
			$search_filter_slug = $atts['search_filter_slug'];
			
			$atts['post_type'] = "custom";
			$atts['search_filter_slug'] = "";
			$atts['taxonomies'] = "";
			
			$search_filter_id = 0;
			
			if ( $post = get_page_by_path( esc_attr($search_filter_slug), OBJECT, 'search-filter-widget' ) )
			{	
				$search_filter_id = (int)$post->ID;
			}
			
			if($search_filter_id!==0)
			{
				$atts['custom_query'] = "search_filter_id=$search_filter_id";
				//$atts['el_class'] .= " search-filter-results-$search_filter_id";
				
				global $search_filter_vc_id;
				$search_filter_vc_id = $search_filter_id;
				add_filter( 'vc_shortcodes_css_class', array($this, 'change_element_class_name'), 1000, 2 );
				add_filter( "vc_grid_request_url", array($this, 'add_query_args_to_request'), 10 );
				
			}
			
		}
		//var_dump($atts);
		
		return $atts;
	}
	
	public function add_query_args_to_request($link)
	{//this is for when ajax is disabled & initial page load, passing query args into the `data-vc-request` attribute of the VC grid shortcode
		foreach($_GET as $key => $value)
		{
			$link = add_query_arg($key, $value, $link);
		}
		
		remove_filter( "vc_grid_request_url", array($this, 'add_query_args_to_request'), 10, 2 );
		
		return $link;
		
	}
	private function get_search_form_options()
	{
		if(empty($this->search_form_options))
		{
			$custom_posts = new WP_Query('post_type=search-filter-widget&post_status=publish&posts_per_page=-1');
			
			$this->search_form_options = array();
			while ($custom_posts->have_posts()) : $custom_posts->the_post();
			$this->search_form_options[html_entity_decode(get_the_title())] = basename(get_permalink());
			endwhile;			
		}
		
		return $this->search_form_options;
	}
	
	public function init_vc_mappings($mapping)
	{
		$formid = "";
		
		//get search forms
		$search_form_options = $this->get_search_form_options();
		
		//var_dump($custom_posts);
		$search_forms_list = array();
		$search_forms_list["Choose a Search Form"] = "";
		$search_forms_list = array_merge($search_forms_list, $search_form_options);
				
		$search_form_param = array(
			"type" => "dropdown",
			'admin_label' => true,
			//'save_always' => true,
			//"holder" => "div",
			"class" => "",
			"heading" => __( "Search Form", "my-text-domain" ),
			"param_name" => "slug",
			"value" => $search_forms_list /*,
			"description" => __( "Search Form Slug", "my-text-domain" ) */
		);
		
		$vc_params = array(
			"name" => __( "Search & Filter Form", "my-text-domain" ),
			"base" => "searchandfilter",
			"icon" => "search-filter-pro-vc-backend-icon",
			'admin_enqueue_css' => array(plugins_url( 'css/vc-backend.css', __FILE__ )),
			"class" => "wn-bild",
			"category" => __( "Content", "my-text-domain"),
			"params" => array(
				$search_form_param
			)
		);
		
		vc_map( $vc_params );
	}
	
	public function vc_add_param_to_grid_settings($wp)
	{
		$search_form_options = $this->get_search_form_options();
		
		//var_dump($custom_posts);
		$search_forms_list = array();
		$search_forms_list["Choose a Search Form"] = "";
		$search_forms_list = array_merge($search_forms_list, $search_form_options);
		
		$search_form_param = array(
			"type" => "dropdown",
			'admin_label' => true,
			'weight' => 1,
			"class" => "",
			"heading" => __( "Search Form", "my-text-domain" ),
			"param_name" => "search_filter_slug",
			"dependency" => array(
				'element' => 'post_type',
				'value' => array('search_filter'),
				//'value' => array('Search & Filter'),
			),
			"value" => $search_forms_list /*,
			"description" => __( "Search Form Slug", "my-text-domain" ) */
		);
		//$search_form_param['weight'] = 100;
		
		$shortcodes = $this->get_vc_shortcodes();
		if(in_array("vc_basic_grid", $shortcodes))
		{
			vc_add_param( 'vc_basic_grid', $search_form_param );
		}
		if(in_array("vc_masonry_grid", $shortcodes))
		{
			vc_add_param( 'vc_masonry_grid', $search_form_param );
		}
		
		
	}
	
	
	public function get_vc_shortcodes()
	{
		if(empty($this->vc_shortcodes))
		{
			$shortcodes_objects = WPBMap::getAllShortCodes();
			$shortcodes = array();
			
			foreach($shortcodes_objects as $shortcode_key => $shortecode)
			{
				array_push($shortcodes, $shortcode_key);
			}
			$this->vc_shortcodes = $shortcodes;
		}
		return $this->vc_shortcodes;
	}
	public function vc_edit_mappings($wp)
	{
		$shortcodes = $this->get_vc_shortcodes();
		/*echo '<pre>'; 
        print_r($shortcodes); 
        echo '</pre>';*/
		
		if(in_array("vc_basic_grid", $shortcodes))
		{
			$this->init_grid_widget("vc_basic_grid");
		}
		if(in_array("vc_masonry_grid", $shortcodes))
		{
			$this->init_grid_widget("vc_masonry_grid");
		}
	}
	public function init_grid_widget($widget_name)
	{
		//add a new option for 
		$post_type_param = WPBMap::getParam( $widget_name, 'post_type' );
		
		if(!$post_type_param)
		{
			return;
		}
		
		$options = $post_type_param['value'];
		
		if((is_array($options))&&(!empty($options)))
		{
			//be nice, remove "search-filter-widget" as an option from teh dropdown
			$index = 0;
			$search_pos = -1;
			foreach($options as $option)
			{
				if($option[0]=="search-filter-widget")
				{
					$search_pos = $index;
				}
				
				$index++;
			}
			
			if($search_pos!==-1)
			{//found, so remove
				unset($options[$search_pos]);
			}
			
			//add a Search & Filter option to the dropdown
			array_push($options, array("search_filter", "Search & Filter"));
			$post_type_param['weight'] = 2; //set the weight of  this param so its above S&F which is set to 1
			$post_type_param['value'] = $options;
			
			//now tell VC the param is updated
			vc_update_shortcode_param( $widget_name, $post_type_param );
		}
				
		//var_dump($post_type_param);
		
		//make sure the taxonomies param does not get shown when teh Search & Filter data source has been set, by adding it to the "exclude" dependency list
		$taxonomies_param = WPBMap::getParam( $widget_name, 'taxonomies' );		
		if(isset($taxonomies_param['dependency']))
		{
			if(isset($taxonomies_param['dependency']['value_not_equal_to']))
			{
				array_push($taxonomies_param['dependency']['value_not_equal_to'], "search_filter");
			}
		}
		//$taxonomies_param['weight'] = 100;
		vc_update_shortcode_param( $widget_name, $taxonomies_param );
		//var_dump($taxonomies_param);
		
		
		//$custom_query_param = WPBMap::getParam( $widget_name, 'custom_query' );		
		//var_dump($custom_query_param);
		
		
	}
	
}
